#!/usr/bin/perl -w

#******************************************************
#	
#	EffiSim
#
#	Calculating the Efficiency of mutation screens for recessive
#	    phenotypes
#
#	Jeremy Silver: silver@wehi.edu.au
#	http://bioinf.wehi.edu.au/software/EffiSim/main.html
#	
#	29 March 2006
#******************************************************

use strict;

# Things to do:
# Get the weights sorted out
# Chi-Squared goodness of fit test
# Some thorough testing!!!

####################################### Declaring variables

my $x = 100;
my $BreedingScheme;
my $d;
my $h;
my $n;
my $k;
my $repeats = 0;
my $fixed;
my $outputfile = "EffiSim.res";
my $screen;
my $tol = 0.00001;
my @ChromLengths = (112,109,87.6,83,86,75,72.4,73,74,75.5,79,61,75,63,69.2,71.5,56.7,59,55.7);
my $NumOfChroms = @ChromLengths;
my $TotalLength;
my $element;
my $chrom;
my @ProbPerChrom;
my @CDF;
my @poissonCDF;
my @MutationSample = ();
my $MutIndex;
my $rep;
my $G1boxes = 0;
my $G2boxes = 0;
my @MutationsSeen;
my @MutationsScreened;
my @G1BoyPatHaplo;
my @G1GirlPatHaplo;
my @G1BoyMatHaplo;
my @G1GirlMatHaplo;
my @G2BoyPatHaplo;
my @G2GirlPatHaplo;
my @G2BoyMatHaplo;
my @G2GirlMatHaplo;
my @G3PupMatHaplo;
my @G3PupPatHaplo;
my @G1GirlMutations;
my @G1BoyMutations;
my @TempPos;
my $mut;
my $RandProb;
my @MutsPerChrom;
my @MutPos;
my @G1BoyMutChrom;
my @G1GirlMutChrom;
my $litter;
my $pup;
my $RecFrac;
my @WhosePups;
my @PupsPerFemale;
my $MaxNonzero;
my $line;
my $paramfile = "";
my @SplitLine;
my $i;
my @params = ();
my @AllParam;
my @CutParam;
my @IntLines;
my @SchemeList;
my @ScreenList;
my @FixedList;
my @DList;
my @HList;
my @TempHList;
my @KList;
my @NList;
my @Ylist;
my $j;
my $t;
my $y = 1;
my @probvec;
my @ICBoxProbs;
my @ICmultiBoxProbs;
my @SupICmultiBoxProbs;
my @SupBCBoxProbs;
my @emptyprobvec;
my @Probabilities = ();
my @Estimates = ();
my @BoxProbs;
my $ex;
my $var;
my @LastProbVec;
my @BCLitterProbs;
my @BCBoxProbs;
my @LitterProbs;
my $numberoflitters;	# Eliminate these variables
my @numberinlitter;
my @lastprobvec;
my $RemainingProb;
my @SupBoxProbs;
my $sum = 0;
my @BeforeHash;
my @parameters;
my $format = "s";
my $sim;
my $SusceptibilityChrom;
my $SusceptibilityLocus;
my $miceused;
my $SusceptibilityLocusIndex;
my @SuscChromPos;
my $Pvalue;
my $df = 0;
my $chsqsum = 0;
my @weights = (1,1,1);
my $CumulativeSum;
my $RemainingCount;
my $MumLength;
my $DadLength;
my $Length;
my $Rate;
my @NonMutChrom;
my @MutChrom;
my $MatSuscState;
my $PatSuscState;
my $Ehat;
my $SDhat;
my @SimResults;
my @simpleEff;

################################ Input parameters and make sure they're 'sensible'

if(@ARGV==0 || @ARGV>7){
	usage();
} else {
	foreach $line (@ARGV){
		@SplitLine = split(//,$line);
		if($SplitLine[0] ne "-"){
			print "\nOptions start with '-'\n";
			usage();
		}
		if(($SplitLine[1] ne "f") && ($SplitLine[1] ne "p") && ($SplitLine[1] ne "n") && ($SplitLine[1] ne "s") && ($SplitLine[1] ne "x") && ($SplitLine[1] ne "o") && ($SplitLine[1] ne "y")){
			print "\nCommands must be from '-f', '-p', '-s' ,'-x' ,'-o' and '-n'\n";
			usage();
		} elsif($SplitLine[1] eq "f"){
			$paramfile = (split(/:/,$line))[1];
		} elsif($SplitLine[1] eq "n"){
			$outputfile = (split(/:/,$line))[1];
		} elsif($SplitLine[1] eq "o"){
			$format = (split(/:/,$line))[1];
			if($format ne "s" && $format ne "l"){
				print "\nFormat must be 's' (short) or 'l' (long)\n";
				exit(1);
			}
		} elsif($SplitLine[1] eq "x"){
			$x = (split(/:/,$line))[1];
			if(!($x =~ /\d+/)){
				print "\nExpected number of mutations must be a number\n";
				exit(1);
			}
			if($x < 0.000001 || $x > 100){
				print "\nExpected number of mutations is bounded: 0.000001<x<100\n";
				exit(1);
			}
		} elsif($SplitLine[1] eq "y"){
			$y = (split(/:/,$line))[1];
			if($y =~ /\d+/){
				if(int($y) != $y){
					print "\nNeed to observe an integer number of homozygotes before we count a mutation as screened\n";
					exit(1);
				}
			} else {
				print "\nRequired number of observed homozygotes to screen a mutation must be a number\n";
				exit(1);
			}
			if($y < 1 || $y > 100){
				print "\nRequired number of observed homozygotes to screen a mutation: 0<y<100\n";
				exit(1);
			}
		} elsif($SplitLine[1] eq "s"){
			$repeats = (split(/:/,$line))[1];
			if(!($repeats =~ /\d+/)){
				print "\nNumber of repeats must be a number\n";
				exit(1);
			}
			if($repeats != int($repeats)){
				print "\nNumber of repeats must be an integer\n";
				exit(1);
			}
			if($repeats<0){
				print "\nNumber of repeats must be a positive integer\n";
				exit(1);
			}
		} elsif($SplitLine[1] eq "p"){
			@params = split(/,|:/,$line);
			if(@params != 9){
				print "\nThe option '-p' requires 8 parameters\n";
				exit(1);
			}
			($Length,$BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @params[1..8];
			if($Length ne "g"){
				if(!($Length =~ /\d+/)){
					print "\nEither specify 'g' for a genome-wide screen\n";
					print " or specify a genetic distance (in cM) for the target region\n";
					exit(1);
				}
				if($Length < 0.0000001 || $Length > 200){
					print "\nLength of target region must lie between 0.0000001 and 200 cM\n";
					exit(1);
				}
			}
			if($BreedingScheme ne "bc" && $BreedingScheme ne "ic"){
				print "\nBreeding Scheme must be either 'bc' or 'ic'\n";
				exit(1);
			}
			if($screen ne "m" && $screen ne "s"){
				print "\nScreen must be either 'm' or 's'\n";
				exit(1);
			}
			if($fixed ne "n" && $fixed ne "r"){
				print "\nFix either 'n' (# pups per G2 female) or 'r' (# pups per G2 male)\n";
				exit(1);
			}
			if($d =~ /\d+/ && $h =~ /\d+/ && $k =~ /\d+/ && $n =~ /\d+/){
				if(!(int($d) == $d && int($h) == $h && int($k) == $k && int($n) == $n)){
					print "\nParameters d, h, n and k must be integers\n";
					exit(1);
				}
			} else {
				print "\nParameters d, h, n and k must be integers\n";
				exit(1);
			}
			if($d > 99 || $d < 1){
				print "\nThe number of G1 pairs is bounded: 0 < d < 100\n";
				exit(1);
			}
			if($k > 99 || $k < 1){
				print "\nThe number of G2 females per G2 male is bounded: 0 < k < 100\n";
				exit(1);
			}
			if($h > 99 || $h < 1){
				print "\nThe number of G1 males per G2 pair is bounded: 0 < h < 100\n";
				exit(1);
			}
			if($n > 99 || $n < 1){
				print "\nThe number of G3 pups per G2 parent is bounded: 0 < n < 100\n";
				exit(1);
			}
			if($Length ne "g"){
				$NumOfChroms = 1;
				@ChromLengths = ($Length);
			}
		}
	}
	@SchemeList = ($BreedingScheme);
	@ScreenList = ($screen);
	@FixedList = ($fixed);
	@DList = ($d);
	@HList = ($h);
	@KList = ($k);
	@NList = ($n);
	@Ylist = ($y);
	if($paramfile eq "" && @params == 0){
		print "\nYou must either specify the parameters in the command line OR give a parameter file\n";
		usage();
	}
	if($paramfile ne "" && @params != 0){
		print "\nYou must either specify the parameters in the command line OR give a parameter file\n";
		print "	but not both\n";
		usage();
	}
}
if($paramfile ne ""){
	open(PARAM,"$paramfile") || die print "\nCannot open $paramfile!!\n";
	@AllParam = <PARAM>;
	close(PARAM);
	@CutParam = ();
	if(@AllParam != 11){
		die print "\nParameter file needs 11 lines of settings ... See the example\n"
	}
	foreach $line (@AllParam){
		push @CutParam, [split(/\s+/,(split(/#/,$line))[0])];
	}
	foreach $element (@{$CutParam[0]}){
		if($element ne "bc" && $element ne "ic"){
			print "\nFirst line of the parameter file should have only 'bc', 'ic' or both\n	 (see example)\n";
			exit(1);
		}
	}
	foreach $element (@{$CutParam[1]}){
		if($element ne "m" && $element ne "s"){
			print "\nSecond line of the parameter file should have only 'm', 's' or both\n	 (see example)\n";
			exit(1);
		}
	}
	foreach $element (@{$CutParam[2]}){
		if($element ne "n" && $element ne "r"){
			print "\nThird line of the parameter file should have only 'r', 'n' or both\n	 (see example)\n";
			exit(1);
		}
	}
	@IntLines = (3,4,5,6,8);
	for($i = 0; $i < 5; $i++){
		$line = $IntLines[$i];
		foreach $element (@{$CutParam[$line]}){
			if(!($element =~ /\d+/)){
				printf("\n%dth line of the parameter file should contain (a) positive integer(s) between 1 and 100\n	 (see example)\n",$line+1);
				exit(1);
			} elsif($element != int($element)){
				printf("\n%dth line of the parameter file should contain (a) positive integer(s) between 1 and 100\n	 (see example)\n",$line+1);
				exit(1);
			} elsif($element < 1 || $element > 99){
				printf("\n%dth line of the parameter file should contain (a) positive integer(s) between 1 and 100\n	 (see example)\n",$line+1);
				exit(1);
			}
		}
	}
	if(@{$CutParam[7]} > 1 || @{$CutParam[7]} == 0){
		print "\nNumber of simulations must be a single number... see the example\n";
		exit(1);
	}
	if(!($CutParam[7][0] =~ /\d+/)){
		print "\nNumber of simulations must be an integer... see the example\n";
		exit(1);
	}
	if(int($CutParam[7][0]) != $CutParam[7][0]){
		print "\nNumber of simulations must be an integer... see the example\n";
		exit(1);
	}
	if($CutParam[7][0]<0){
		print "\nNumber of simulations cannot be negative... see the example\n";
		exit(1);
	}
	if($CutParam[9][0] ne "g"){
		if(!($CutParam[9][0] =~ /\d+/)){
			print "\nEither specify 'g' for a genome-wide screen\n";
			print " or specify a genetic distance (in cM) for the target region\n";
			exit(1);
		}
		if($CutParam[9][0] < 0.0000001 || $CutParam[9][0] > 200){
			print "\nLength of target region must lie between 0.0000001 and 200 cM\n";
			exit(1);
		}
	}
	if(@{$CutParam[10]} != 3){
		print "\nNeed average costs for THREE generations... see the example\n";
		exit(1);
	}		
	for($i = 0; $i < 3; $i++){
		$element = $CutParam[10][$i];
		if(!($element =~ /\d+/)){
			print "\nWeights must be numbers... see the example\n";
			exit(1);
		}
		if($element<0){
			print "\nWeights cannot be negative... see the example\n";
			exit(1);
		}
	}
	@SchemeList = @{$CutParam[0]};
	@ScreenList = @{$CutParam[1]};
	@FixedList = @{$CutParam[2]};
	@DList = @{$CutParam[3]};
	@HList = @{$CutParam[4]};
	@KList = @{$CutParam[5]};
	@NList = @{$CutParam[6]};
	$repeats = $CutParam[7][0];
	@Ylist = @{$CutParam[8]};
	$Length = $CutParam[9][0];
	@weights = @{$CutParam[10]}[0..2];
	if($Length ne "g"){
		$NumOfChroms = 1;
		@ChromLengths = ($Length);
	}
}
########################################################## Prepare for Poisson distribution

$Rate = $x;
$i = 0;
$CumulativeSum = 0;
@poissonCDF = ();
while($CumulativeSum < 0.999999){
	$CumulativeSum += ($Rate**$i) * exp(-$Rate) / factorial($i);
	push @poissonCDF, $CumulativeSum;
	$i++;
}
if($CumulativeSum > 1){
	die print "\nProbabilities should not add up to more than 1\n";
}
$TotalLength = 0;
foreach $element (@ChromLengths){
	$TotalLength += $element;
}
@ProbPerChrom = ();
for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
	push @ProbPerChrom, $ChromLengths[$chrom]/$TotalLength;
}
@CDF = ($ProbPerChrom[0]);
for($chrom = 1; $chrom < $NumOfChroms; $chrom++){
	push @CDF, $CDF[-1] + $ProbPerChrom[$chrom];
}

########################################################## Print the results

sub PrintResults{

	my($BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$y) = @_[0..7];
	my @Estimates = @{$_[8]};
	my @Probabilities = @{$_[9]};
	my($chsqsum,$df,$Ehat,$SDhat) = @_[10..13];
	my $simpleMean;
	my $simpleVar;
	$x = $Rate;
	if($#_ == 14){
		$simpleMean = $_[14][0];
		$simpleVar = $_[14][1];
	}
	my $miceused = CountMice($BreedingScheme,$screen,$fixed,$d,$h,$k,$n,[@weights]);
	my $factor;
	if($BreedingScheme eq "bc"){
		$factor = $repeats*$d*$x;
	} else {
		$factor = $repeats*$d*2*$x;
	}
	if($format eq "l"){
		if($BreedingScheme eq "bc"){
			print OUT "# G1 males = $d, # G2 females/G2 male = $k";
		} else {
			print OUT "# G1 pairs = $d, # G2 pairs/# G1 pairs = $h, # G2 females/G2 male = $k";
		}
		if($fixed eq "n"){
			print OUT ", # G3 pups per G2 female = $n\n";
		} else {
			print OUT ", # G3 pups per G2 male = $n\n";
		}
		print OUT "breeding scheme = $BreedingScheme, screen for ";
		if($screen eq "m"){
			print OUT "recessive mutations\n";
		} else {
			print OUT "recessive suppressors of recessive mutations\n";
		}
		if($repeats>0){
			print OUT "# of simulations = $repeats, expected # of mutations per G1 pup = $x\n";
			print OUT "Average cost per mouse in G1,	G2,	G3 (respectively)\n";
			print OUT "                          $weights[0],	$weights[1],	$weights[2]\n";
		} else {
			print OUT "No simulations, expected # of mutations per G1 pup = $x\n";
			print OUT "Average cost per mouse in G1,	G2,	G3 (respectively)\n";
			print OUT "                          $weights[0],	$weights[1],	$weights[2]\n";
		}
		if($y>1){
			print OUT "A mutation must be homozygous in at least $y G3 pups to be screened\n";
		} else {
			print OUT "It is sufficient to observe a single G3 homozygote to to be screen a mutation\n";
		}
		if($Length eq "g"){
			print OUT "Simulations represent a genome-wide screen\n";
		} else {
			print OUT "Simulations represent a regional screen, with a target length of $Length cM\n";
		}		
		$sum = 0;
		for($mut = $#Probabilities; $mut >= $y; $mut--){
			$sum += $Probabilities[$mut];
		}
		if($repeats>0){
			$CumulativeSum = 0;
#			print OUT "Seen in m G3 mice		Obs(# of mut'ns)		Exp(# of mut'ns)		(O-E)^2/E		sum((O-E)^2/E)\n";
			print OUT "Seen in m G3 mice		Obs(# of mut'ns)		Exp(# of mut'ns)\n";
			for($mut = 0; $mut < @Estimates; $mut++){
				$RemainingProb = 0;
				if($mut != $#Estimates){
					for($j = $mut+1; $j<@Estimates; $j++){
						$RemainingProb += $Probabilities[$j];
					}
				}
				if($BreedingScheme eq "bc"){
					if($d*$x*$repeats*$RemainingProb >= 5){				
						$CumulativeSum += (($Estimates[$mut] - $d*$x*$repeats*$Probabilities[$mut])**2)/($d*$x*$repeats*$Probabilities[$mut]);
						printf OUT ("		%d					%3.2f					%3.2f\n",$mut,$Estimates[$mut]/$repeats,$d*$x*$Probabilities[$mut]);
	#					printf OUT ("		%d					%d					%3.2f				%3.2f				%3.2f\n",$mut,$Estimates[$mut],$d*$x*$repeats*$Probabilities[$mut],(($Estimates[$mut] - $d*$x*$repeats*$Probabilities[$mut])**2)/($d*$x*$repeats*$Probabilities[$mut]),$CumulativeSum);
					} else {
						$RemainingProb += $Probabilities[$mut];
						$RemainingCount = 0;
						for($i = $mut; $i < @Estimates; $i++){
							$RemainingCount += $Estimates[$i];
						}
						$df = $mut;
						$CumulativeSum += (($RemainingCount - $d*$x*$repeats*$RemainingProb)**2)/($d*$x*$repeats*$RemainingProb);
						printf OUT ("		>%d					%3.2f					%3.2f\n",$mut - 1,$Estimates[$mut]/$repeats,$d*$x*$Probabilities[$mut]);
	#					printf OUT ("		>%d					%d					%3.2f				%3.2f				%3.2f\n",$mut - 1,$RemainingCount,$d*$x*$repeats*$RemainingProb,(($RemainingCount - $d*$x*$repeats*$RemainingProb)**2)/($d*$x*$repeats*$RemainingProb),$CumulativeSum);
						$mut = @Estimates;
					}
				} else {
					if(2*$d*$x*$repeats*$RemainingProb >= 5){							
						$CumulativeSum += (($Estimates[$mut] - 2*$d*$x*$repeats*$Probabilities[$mut])**2)/(2*$d*$x*$repeats*$Probabilities[$mut]);
						printf OUT ("		%d					%3.2f					%3.2f\n",$mut,$Estimates[$mut]/$repeats,$d*2*$x*$Probabilities[$mut]);
	#					printf OUT ("		%d					%d					%3.2f				%3.2f				%3.2f\n",$mut,$Estimates[$mut],$d*2*$x*$repeats*$Probabilities[$mut],(($Estimates[$mut] - 2*$d*$x*$repeats*$Probabilities[$mut])**2)/(2*$d*$x*$repeats*$Probabilities[$mut]),$CumulativeSum);
					} else {
						$RemainingProb += $Probabilities[$mut];
						$RemainingCount = 0;
						for($i = $mut; $i < @Estimates; $i++){
							$RemainingCount += $Estimates[$i];
						}
						$df = $mut;
						$CumulativeSum += (($RemainingCount - 2*$d*$x*$repeats*$RemainingProb)**2)/(2*$d*$x*$repeats*$RemainingProb);
						printf OUT ("		>%d					%3.2f					%3.2f\n",$mut - 1,$Estimates[$mut]/$repeats,2*$d*$x*$Probabilities[$mut]);
	#					printf OUT ("		>%d					%d					%3.2f				%3.2f				%3.2f\n",$mut - 1,$RemainingCount,2*$d*$x*$repeats*$RemainingProb,(($RemainingCount - 2*$d*$x*$repeats*$RemainingProb)**2)/(2*$d*$x*$repeats*$RemainingProb),$CumulativeSum);
						$mut = @Estimates;
					}
				}				
			}
#			printf OUT ("Chi-squ. sum = %3.3f, df = %d\n",$CumulativeSum,$df);
			if($Ehat==0){
				printf OUT ("Eff(obs) = %3.2f,	SD(obs) = %3.2f,	CostPerMut(obs) = ###, cost = %3.2f\n",$Ehat,$SDhat,$miceused);
			} else {
				printf OUT ("Eff(obs) = %3.2f,	SD(obs) = %3.2f,	CostPerMut(obs) = %3.2f, cost = %3.2f\n",$Ehat,$SDhat,$miceused/$Ehat,$miceused);
			}
			if($y==1){
				printf OUT ("Eff(exp) = %3.2f,	CostPerMut(obs) = %3.2f\n",$simpleMean,$miceused/$simpleMean);
			} else {
				if($BreedingScheme eq "bc"){
					if($sum == 0){
						printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = ###\n",$sum*$d*$x);
					} else {
						printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = %3.2f\n",$sum*$d*$x,$miceused/($sum*$d*$x));
					}
				} else {
					if($sum == 0){
						printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = ###\n",2*$sum*$d*$x);
					} else {
						printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = %3.2f\n",2*$sum*$d*$x,$miceused/(2*$sum*$d*$x));
					}
				}
			}
		} else {
			print OUT "Seen in m G3 mice		Exp(# of mut'ns)\n";
			$MaxNonzero = 0;
			for($mut = 0; $mut < @Probabilities; $mut++){
				if($BreedingScheme eq "bc"){
					printf OUT ("		%d					%3.2f\n",$mut,$x*$d*$Probabilities[$mut]);
				} else {
					printf OUT ("		%d					%3.2f\n",$mut,2*$x*$d*$Probabilities[$mut]);
				}
				if($Probabilities[$mut]<$tol){
					$MaxNonzero = $mut;
					$mut = @Probabilities;
				}
			}
			if($MaxNonzero > 0){
				$RemainingProb = 0;
				for($i = $MaxNonzero; $i<@Probabilities; $i++){
					$RemainingProb += $Probabilities[$i];
				}
			} else {
				$RemainingProb = 0;
				$MaxNonzero = @Probabilities - 1;
			}
			if($BreedingScheme eq "bc"){
				printf OUT ("		>%d					%3.2f\n",$MaxNonzero,$x*$RemainingProb);
			} else {
				printf OUT ("		>%d					%3.2f\n",$MaxNonzero,2*$x*$RemainingProb);
			}
			if($y==1){
				printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = %3.2f, cost = %3.2f\n",$simpleMean,$miceused/$simpleMean,$miceused);
			} else {
				if($BreedingScheme eq "bc"){
					printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = %3.2f, cost = %3.2f\n",$sum*$d*$x,$miceused/($sum*$d*$x),$miceused);
				} else {
					printf OUT ("Eff(exp) = %3.2f,	CostPerMut(exp) = %3.2f, cost = %3.2f\n",$sum*$d*$x,$miceused/(2*$sum*$d*$x),$miceused);
				}
			}
		}
	} else {
		$sum = 0;
		for($mut = $#Probabilities; $mut >= $y; $mut--){
			$sum += $Probabilities[$mut];
		}
		if($repeats>0){
			$sim = 0;
			for($mut = $#Estimates; $mut >= $y; $mut--){
				$sim += $Estimates[$mut];
			}
			for($mut = 0; $mut < @Estimates; $mut++){
				$RemainingProb = 0;
				if($mut != $#Estimates){
					for($j = $mut+1; $j<@Estimates; $j++){
						$RemainingProb += $Probabilities[$j];
					}
				}
				if($BreedingScheme eq "bc"){
					if($d*$x*$repeats*$RemainingProb >= 5){				
						$CumulativeSum += (($Estimates[$mut] - $d*$x*$repeats*$Probabilities[$mut])**2)/($d*$x*$repeats*$Probabilities[$mut]);
					} else {
						$RemainingProb += $Probabilities[$mut];
						$RemainingCount = 0;
						for($i = $mut; $i < @Estimates; $i++){
							$RemainingCount += $Estimates[$i];
						}
						$df = $mut;
						$CumulativeSum += (($RemainingCount - $d*$x*$repeats*$RemainingProb)**2)/($d*$x*$repeats*$RemainingProb);
						$mut = @Estimates;
					}
				} else {
					if(2*$d*$x*$repeats*$RemainingProb >= 5){							
						$CumulativeSum += (($Estimates[$mut] - 2*$d*$x*$repeats*$Probabilities[$mut])**2)/(2*$d*$x*$repeats*$Probabilities[$mut]);
					} else {
						$RemainingProb += $Probabilities[$mut];
						$RemainingCount = 0;
						for($i = $mut; $i < @Estimates; $i++){
							$RemainingCount += $Estimates[$i];
						}
						$df = $mut;
						$CumulativeSum += (($RemainingCount - 2*$d*$x*$repeats*$RemainingProb)**2)/(2*$d*$x*$repeats*$RemainingProb);
						$mut = @Estimates;
					}
				}
			}
			if($BreedingScheme eq "bc"){
				if($Ehat==0){
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d	%d		%3.2f	*****	%3.3f		%3.3f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,$miceused,$Ehat,$SDhat);
				} else {
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d	%d		%3.2f	%3.2f	%3.3f		%3.3f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,$miceused,$miceused/$Ehat,$Ehat,$SDhat);
				}
#				printf OUT ("%s	%s	%s	%d	%d	%d	%d	%d	%d	%d		%3.2f	%3.2f	%3.2f	%3.2f 	%3.3f	%d	%3.2f 	%3.2f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,$sum*$d*$x,$sim/$repeats,$miceused,$sum*$d*$x/$miceused,$CumulativeSum,$df,$Ehat,$SDhat);
				if($y==1){
					printf OUT ("	%3.3f		%3.3f\n",$simpleMean,$miceused/$simpleMean);
				} else {
					printf OUT "	%3.3f		%3.3f\n",$sum*$d*$x,$miceused/($sum*$d*$x);
				}
			} else {
				if($Ehat==0){
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d	%d		%3.2f	*****	%3.3f		%3.3f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,$miceused,$Ehat,$SDhat);
				} else {
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d	%d		%3.2f	%3.2f	%3.3f		%3.3f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,$miceused,$miceused/$Ehat,$Ehat,$SDhat);
				}
#				printf OUT ("%s	%s	%s	%d	%d	%d	%d	%d	%d	%d		%3.2f	%3.2f	%3.2f	%3.2f 	%3.3f	%d	%3.2f 	%3.2f",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$repeats,2*$sum*$d*$x,$sim/$repeats,$miceused,2*$sum*$d*$x/$miceused,$CumulativeSum,$df,$Ehat,$SDhat);
				if($y==1){
					printf OUT ("	%3.3f		%3.3f\n",$simpleMean,$miceused/$simpleMean);
				} else {
					printf OUT ("	%3.3f		%3.3f\n",2*$sum*$d*$x,$miceused/(2*$sum*$d*$x));
				}
			}
		} else {
			if($BreedingScheme eq "bc"){
				if($y==1){
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d		%3.2f		%3.2f		%3.5f\n",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$simpleMean,$miceused,$miceused/$simpleMean);
				} else {
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d		%3.2f		%3.2f		%3.5f\n",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$sum*$d*$x,$miceused,$miceused/($sum*$d*$x));
				}
			} else {
				if($y==1){
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d		%3.2f		%3.2f		%3.5f\n",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,$simpleMean,$miceused,$miceused/$simpleMean);
				} else {
					printf OUT ("%s	%s	%s	%d	%d	%d	%d	%3.2f	%d		%3.2f		%3.2f		%3.5f\n",$BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$x,$y,2*$sum*$d*$x,$miceused,$miceused/(2*$sum*$d*$x));
				}
			}
		}
	}
}

################################ Distribute the work

open(OUT, ">$outputfile");
if($format eq "s"){
	if($repeats>0){
		print OUT "B	S	F	d	h	k	n	x		y	reps	Cost	CM(obs)	Eff(obs)	SD(obs)	Eff(exp)	CM(exp)\n";
	} else {
		print OUT "B	S	F	d	h	k	n	x		y		Eff(exp)	Cost		CM(exp)\n";
	}		
}
@parameters = ($SchemeList[0],$ScreenList[0],$FixedList[0],$DList[0],$HList[0],$KList[0],$NList[0]);
foreach $BreedingScheme (@SchemeList){
	foreach $screen (@ScreenList){
		foreach $fixed (@FixedList){
			if($BreedingScheme eq "bc"){
				@TempHList = (1);
			} else {
				@TempHList = @HList;
			}
			foreach $h (@TempHList){
				foreach $k (@KList){
					foreach $n (@NList){
						foreach $d (@DList){
							@parameters = ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n);
							@Probabilities = Calculate(@parameters);
							$sum = 0;
							foreach $element (@Probabilities){
								$sum += $element;
							}
							if(abs($sum - 1) > $tol){			## This shouldn't happen but we'll catch it if it does
								print "\nSum of probabilities was $sum and not 1\n";
								exit(1);
							}
							if($repeats>0){
								@SimResults = Simulate(@parameters);
								$Ehat = $SimResults[0];
								$SDhat = $SimResults[1];
								@Estimates = @{$SimResults[2]};
								($chsqsum, $df) = GoodnessOfFit([@Probabilities],[@Estimates],[@parameters]);
							}
							foreach $y (@Ylist){
								if($y == 1){
									@parameters = ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n);
									@simpleEff = simple(@parameters);
									@parameters = ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$y,[@Estimates],[@Probabilities],$chsqsum,$df,$Ehat,$SDhat,[@simpleEff]);
									PrintResults(@parameters);
								} else {
									@parameters = ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n,$y,[@Estimates],[@Probabilities],$chsqsum,$df,$Ehat,$SDhat);
									PrintResults(@parameters);
								}
							}
						}
					}
				}
			}
		}
	}
}

close(OUT);

################################################## Simulation section

sub Simulate{

	my ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @_;
	my @MutationsSeen;
	my @CountMutationsSeen = ();
	my $mean = 0;
	my $sd = 0;
	my $MutCounter = 0;
	if($fixed eq "n"){
		if($BreedingScheme eq "bc"){	### $MutationsSeen[$i] = # mutations seen in $i mice
			@MutationsSeen = (0) x ($n*$k+1);		# bc and fixed
		} else {
			@MutationsSeen = (0) x ($h*$n*$k+1);	# ic and fixed
		}
	} else {
		if($BreedingScheme eq "bc"){				# bc and flexible
			@MutationsSeen = (0) x ($n+1);
		} else {
			@MutationsSeen = (0) x ($h*$n+1);		# ic and flexible
		}
	}
	for($rep = 0; $rep<$repeats; $rep++){
		for($G1boxes = 0; $G1boxes < $d; $G1boxes++){
			$RandProb = rand();
			$x = -1;
			for($mut = 0; $mut < @poissonCDF; $mut++){
				if($RandProb <= $poissonCDF[$mut]){
					$x = $mut;
					$mut = @poissonCDF;
				}
			}
			if($x == -1){
				print "x = $mut with pvalue $CumulativeSum, RandProb was $RandProb\n";
				$x = @poissonCDF + 1;
			}
			if($x > 0){
				if($BreedingScheme eq "bc"){
					@MutationsScreened = (0) x $x;
					@MutationSample = ();
					for($mut = 0; $mut < $x; $mut++){
						$RandProb = rand();
						for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
							if($RandProb < $CDF[$chrom]){
								push @MutationSample, $chrom;
								$chrom = $NumOfChroms;
							}
						}
					}
					@MutsPerChrom = (0) x $NumOfChroms;
					for($mut = 0; $mut < $x; $mut++){
						$MutsPerChrom[$MutationSample[$mut]]++;
					}
					@MutPos = ();
					@TempPos = ();
					for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
						@TempPos = ();
						for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
							push @TempPos, rand($ChromLengths[$chrom]);
						}
						@TempPos = sort numerically @TempPos;
						push @MutPos, [@TempPos];
					}
					if($screen eq "s"){
						$RandProb = rand();
						for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
							if($RandProb < $CDF[$chrom]){
								$SusceptibilityChrom = $chrom;
								$chrom = $NumOfChroms;
							}
						}
						$SusceptibilityLocus = rand($ChromLengths[$SusceptibilityChrom]);
						$SusceptibilityLocusIndex = 0;
						for($mut = 0; $mut < $MutsPerChrom[$SusceptibilityChrom] ; $mut ++){
							if($MutPos[$SusceptibilityChrom][$mut] < $SusceptibilityLocus){
								$SusceptibilityLocusIndex++;
							} else {
								$mut = $MutsPerChrom[$SusceptibilityChrom];
							}
						}
						@SuscChromPos = (@{$MutPos[$SusceptibilityChrom]}[0..$SusceptibilityLocusIndex - 1],$SusceptibilityLocus,@{$MutPos[$SusceptibilityChrom]}[$SusceptibilityLocusIndex..$MutsPerChrom[$SusceptibilityChrom] - 1]);
					}
					@G1BoyMatHaplo = ();
					@G1BoyPatHaplo = ();
					for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
						@MutChrom = ();
						@NonMutChrom = ();
						for($mut = 0; $mut<$MutsPerChrom[$chrom]; $mut++){
							push @MutChrom, 1;
							push @NonMutChrom, 0;
						}
						push @G1BoyPatHaplo, [@MutChrom];
						push @G1BoyMatHaplo, [@NonMutChrom];
					}
					@G1GirlMatHaplo = @G1BoyMatHaplo;
					@G1GirlPatHaplo = @G1BoyMatHaplo;
					if($fixed eq "r"){
						@WhosePups = ();
						for($pup = 0; $pup<$n; $pup++){
							push @WhosePups, int(rand($k));
						}
						@PupsPerFemale = (0) x $k;
						for($pup = 0; $pup<$n; $pup++){
							$PupsPerFemale[$WhosePups[$pup]]++;
						}
					} else {
						@PupsPerFemale = ($n) x $k;
					}
					for($litter = 0; $litter < $k; $litter++){
						@G2GirlPatHaplo = segregate(([@G1BoyMatHaplo],[@G1BoyPatHaplo]));
						@G2GirlMatHaplo = @G1BoyMatHaplo;
						for($pup = 0; $pup<$PupsPerFemale[$litter]; $pup++){
							if($screen eq "s"){
								@G3PupPatHaplo = segregate(([@G1BoyMatHaplo],[@G1BoyPatHaplo]));
								@G3PupMatHaplo = segregate(([@G2GirlMatHaplo],[@G2GirlPatHaplo]));
								$MatSuscState = int(2*rand());
								$PatSuscState = int(2*rand());
								if($MatSuscState == 1 && $PatSuscState == 1){
									$MutIndex = 0;
									for($chrom = 0; $chrom<$NumOfChroms ; $chrom++){
										if($MutsPerChrom[$chrom]>0){
											for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
												if($G3PupPatHaplo[$chrom][$mut]==1 && $G3PupMatHaplo[$chrom][$mut]==1){
													$MutationsScreened[$MutIndex]++;
												}
												$MutIndex++;
											}
										}
									}
								}
							} else {
								@G3PupPatHaplo = segregate(([@G1BoyMatHaplo],[@G1BoyPatHaplo]));
								@G3PupMatHaplo = segregate(([@G2GirlMatHaplo],[@G2GirlPatHaplo]));
								$MutIndex = 0;
								for($chrom = 0; $chrom<$NumOfChroms ; $chrom++){
									if($MutsPerChrom[$chrom]>0){
										for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
											if($G3PupPatHaplo[$chrom][$mut]==1 && $G3PupMatHaplo[$chrom][$mut]==1){
												$MutationsScreened[$MutIndex]++;
											}
											$MutIndex++;
										}
									}
								}
							}
						}
					}
				}else{  ## We're dealing with the intercross
					@MutationSample = ();
					@MutationsScreened = (0) x (2*$x);
					for($mut = 0; $mut < 2*$x; $mut++){
						$RandProb = rand();
						for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
							if($RandProb < $CDF[$chrom]){
								push @MutationSample, $chrom;
								$chrom = $NumOfChroms;
							}
						}
					}
					@MutsPerChrom = (0) x $NumOfChroms;
					for($mut = 0; $mut < 2*$x; $mut++){
						$MutsPerChrom[$MutationSample[$mut]]++;
					}
					@MutPos = ();
					@TempPos = ();
					for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
						@TempPos = ();
						for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
							push @TempPos, rand($ChromLengths[$chrom]);
						}
						@TempPos = sort numerically @TempPos;
						push @MutPos, [@TempPos];
					}
					if($screen eq "s"){
						$RandProb = rand();
						for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
							if($RandProb < $CDF[$chrom]){
								$SusceptibilityChrom = $chrom;
								$chrom = $NumOfChroms;
							}
						}
						$SusceptibilityLocus = rand($ChromLengths[$SusceptibilityChrom]);
						$SusceptibilityLocusIndex = 0;
						for($mut = 0; $mut < $MutsPerChrom[$SusceptibilityChrom] ; $mut ++){
							if($MutPos[$SusceptibilityChrom][$mut] < $SusceptibilityLocus){
								$SusceptibilityLocusIndex++;
							} else {
								$mut = $MutsPerChrom[$SusceptibilityChrom];
							}
						}
						@SuscChromPos = (@{$MutPos[$SusceptibilityChrom]}[0..$SusceptibilityLocusIndex - 1],$SusceptibilityLocus,@{$MutPos[$SusceptibilityChrom]}[$SusceptibilityLocusIndex..$MutsPerChrom[$SusceptibilityChrom] - 1]);
					}
					@G1BoyMutations = SmplWORplmnt((2*$x,[(((0) x $x),((1) x $x))]));
					@G1GirlMutations = ();
					foreach $element (@G1BoyMutations){
						push @G1GirlMutations, 1-$element;
					}
					@G1BoyPatHaplo = ();
					@G1GirlPatHaplo = ();
					@G1BoyMatHaplo = ();
					@G1GirlMatHaplo = ();
					$MutIndex = 0;
					for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
						@G1BoyMutChrom = ();
						@NonMutChrom = ();
						@G1GirlMutChrom = ();
						for($mut = 0; $mut<$MutsPerChrom[$chrom]; $mut++){
							push @G1BoyMutChrom, $G1BoyMutations[$MutIndex];
							push @G1GirlMutChrom, $G1GirlMutations[$MutIndex];
							push @NonMutChrom, 0;
							$MutIndex++;
						}
						push @G1BoyPatHaplo, [@G1BoyMutChrom];
						push @G1GirlPatHaplo, [@G1GirlMutChrom];
						push @G1BoyMatHaplo, [@NonMutChrom];
						push @G1GirlMatHaplo, [@NonMutChrom];
					}
					for($G2boxes = 0; $G2boxes<$h; $G2boxes++){
						@G2BoyMatHaplo = segregate(([@G1GirlMatHaplo],[@G1GirlPatHaplo]));
						@G2BoyPatHaplo = segregate(([@G1BoyMatHaplo],[@G1BoyPatHaplo]));
						if($fixed eq "r"){
							@WhosePups = ();
							for($pup = 0; $pup<$n; $pup++){
								push @WhosePups, int(rand($k));
							}
							@PupsPerFemale = (0) x $k;
							for($pup = 0; $pup<$n; $pup++){
								$PupsPerFemale[$WhosePups[$pup]]++;
							}
						} else {
							@PupsPerFemale = ($n) x $k;
						}
						for($litter = 0; $litter < $k ; $litter++){
							@G2GirlMatHaplo = segregate(([@G1GirlMatHaplo],[@G1GirlPatHaplo]));
							@G2GirlPatHaplo = segregate(([@G1BoyMatHaplo],[@G1BoyPatHaplo]));
							for($pup = 0; $pup<$PupsPerFemale[$litter]; $pup++){
								if($screen eq "s"){
									@G3PupPatHaplo = segregate(([@G2BoyMatHaplo],[@G2BoyPatHaplo]));
									@G3PupMatHaplo = segregate(([@G2GirlMatHaplo],[@G2GirlPatHaplo]));
									$MatSuscState = int(2*rand());
									$PatSuscState = int(2*rand());
									if($MatSuscState == 1 && $PatSuscState == 1){
										$MutIndex = 0;
										for($chrom = 0; $chrom<$NumOfChroms ; $chrom++){
											for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
												if($G3PupPatHaplo[$chrom][$mut]==1 && $G3PupMatHaplo[$chrom][$mut]==1){
													$MutationsScreened[$MutIndex]++;
												}
												$MutIndex++;
											}
										}
									}
								} else {
									@G3PupMatHaplo = segregate(([@G2GirlMatHaplo],[@G2GirlPatHaplo]));
									@G3PupPatHaplo = segregate(([@G2BoyMatHaplo],[@G2BoyPatHaplo]));
									$MutIndex = 0;
									for($chrom = 0; $chrom<$NumOfChroms ; $chrom++){
										if($MutsPerChrom[$chrom]>0){
											for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
												if($G3PupPatHaplo[$chrom][$mut]==1 && $G3PupMatHaplo[$chrom][$mut]==1){
													$MutationsScreened[$MutIndex]++;
												}
												$MutIndex++;
											}
										}
									}
								}
							}
						}
					}
				}
				for($mut = 0; $mut< @MutationsScreened ; $mut++){
					$MutationsSeen[$MutationsScreened[$mut]]++;
				}
				@MutsPerChrom = (0) x $NumOfChroms;
				for($mut = 0; $mut< @MutationsScreened ; $mut++){
					if($MutationsScreened[$mut] >= $y){
						$MutCounter++;
					}
				}
			}
			push @CountMutationsSeen, $MutCounter;
			$MutCounter = 0;
		}
	}
	$mean = 0;
	for($rep = 0; $rep < @CountMutationsSeen; $rep++){
		$mean += $CountMutationsSeen[$rep]/@CountMutationsSeen;
	}
	$mean *= $d;
	if($repeats > 1){
		$var = 0;
		for($rep = 0; $rep < @CountMutationsSeen; $rep++){
			$var += (($CountMutationsSeen[$rep] - $mean)**2)/(@CountMutationsSeen - 1);
		}
		$sd	= $var**0.5;
	} else {
		$sd = 0;
	}
	for($mut = $#MutationsSeen; $mut > -1; $mut--){
		if($MutationsSeen[$mut]>0){
			$MaxNonzero = $mut+1;
			$mut = -1;
		}
	}
	return($mean,$sd,[@MutationsSeen]);
}	

############################################# Subroutines for the simulations

sub numerically {$a <=> $b} # sort in increasing numerical order

sub segregate{

	my @MatHaplo = @{$_[0]};
	my @PatHaplo = @{$_[1]};
	my @OffspringHaplo = ();
	my $chrom;
	my @ChromHaplo;
	my $mut;
	my $WhichChrom;
	my $RecFrac;
	my $RandProb;
	for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
		$WhichChrom = int(rand(2));
		@ChromHaplo = ();
		for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
			if($WhichChrom==0){
				push @ChromHaplo, $MatHaplo[$chrom][$mut];
			} else {
				push @ChromHaplo, $PatHaplo[$chrom][$mut];
			}
			if($mut+1<$MutsPerChrom[$chrom]){
				$RecFrac = 0.5*(1-exp(-2*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100));
#				$RecFrac = 0.5*(exp(4*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100) - 1)/(exp(4*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100) + 1);
#				$RecFrac = 0.5;
				$RandProb = rand();
				if($RandProb < $RecFrac){
					$WhichChrom = 1 - $WhichChrom;
				}
			}
		}
		push @OffspringHaplo, [@ChromHaplo];
	}
	return(@OffspringHaplo)	
}

sub segregateSusc{

	my @MatHaplo = @{$_[0]};
	my @PatHaplo = @{$_[1]};
	my $PatSuscAll = int(rand(2));
	my $MatSuscAll = 1 - $PatSuscAll;
	my @OffspringHaplo = ();
	my $chrom;
	my $RecFrac;
	my $RandProb;
	my @ChromHaplo;
	my $mut;
	my $WhichChrom;
	my @MatSuscHaplo = (@{$MatHaplo[$SusceptibilityChrom]}[0..$SusceptibilityLocusIndex - 1],$MatSuscAll,@{$MatHaplo[$SusceptibilityChrom]}[$SusceptibilityLocusIndex..$MutsPerChrom[$SusceptibilityChrom] - 1]);
	my @PatSuscHaplo = (@{$PatHaplo[$SusceptibilityChrom]}[0..$SusceptibilityLocusIndex - 1],$PatSuscAll,@{$PatHaplo[$SusceptibilityChrom]}[$SusceptibilityLocusIndex..$MutsPerChrom[$SusceptibilityChrom] - 1]);
	for($chrom = 0; $chrom < $NumOfChroms; $chrom++){
		$WhichChrom = int(rand(2));
		@ChromHaplo = ();
		if($chrom !=$SusceptibilityChrom){
			for($mut = 0; $mut < $MutsPerChrom[$chrom]; $mut++){
				if($WhichChrom==0){
					push @ChromHaplo, $MatHaplo[$chrom][$mut];
				} else {
					push @ChromHaplo, $PatHaplo[$chrom][$mut];
				}
				if($mut+1<$MutsPerChrom[$chrom]){
					$RecFrac = 0.5*(1-exp(-2*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100));
#					$RecFrac = 0.5*(exp(4*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100) - 1)/(exp(4*($MutPos[$chrom][$mut+1] - $MutPos[$chrom][$mut])/100) + 1);
#					$RecFrac = 0.5;
					$RandProb = rand();
					if($RandProb < $RecFrac){
						$WhichChrom = 1 - $WhichChrom;
					}
				}
			}
		} else {	# The susceptibility chromosome case
			for($mut = 0; $mut < $MutsPerChrom[$chrom] + 1; $mut++){
				if($WhichChrom==0){
					push @ChromHaplo, $MatSuscHaplo[$mut];
				} else {
					push @ChromHaplo, $PatSuscHaplo[$mut];
				}
				if($mut+1<$MutsPerChrom[$chrom] + 1){
					$RecFrac = 0.5*(1-exp(-2*($SuscChromPos[$mut+1] - $SuscChromPos[$mut])/100));
#					$RecFrac = 0.5*(exp(4*($SuscChromPos[$mut+1] - $SuscChromPos[$mut])/100) - 1)/(exp(4*($SuscChromPos[$mut+1] - $SuscChromPos[$mut])/100) + 1);
#					$RecFrac = 0.5;
					$RandProb = rand();
					if($RandProb < $RecFrac){
						$WhichChrom = 1 - $WhichChrom;
					}
				}
			}
		}
		push @OffspringHaplo, [@ChromHaplo];
	}
	return(@OffspringHaplo)	
}


sub SmplWORplmnt{

	my @selection = @{$_[1]};
	my $n = $_[0];
	my $i;
	my $k = @selection;
	my @sample = ();
	my $index;
	if($n > @selection){
		die print "Cannot sample without replacement and take more than n samples\n";
	} else {
		for($i = 0; $i<$n; $i++){
			$index = int(rand($k));
			push @sample, $selection[$index];
			@selection = (@selection[0..$index-1],@selection[$index+1..$k]);
			$k--;
		}
	}
	return(@sample);
}

sub usage{
	print "Usage:\nperl EffiSim.pl -f:par.txt [-n:res.txt] [-o:s/l]\n\n";
	print "	the -f option specifies a parameter file (see documentation)\n";
	print "	the -n option specifies where the results go, default is EffiSim.res\n";
	print "	the -o option specifies the type of output short or long ('s' or 'l')\n";
	print "\n			OR...\n\n";
	print "perl EffiSim.pl -p:L,sch,scr,fix,d,h,k,n [-s:reps] [-x:mt] [-n:res.txt] [-o:s/l]\n\n";
	print "	L is the either 'g' for a genome-wide screen\n";
	print "	 or it is the length (in cM) of the target interval in a regional screen\n";
	print "	sch is the breeding scheme: either 'bc' or 'ic' - backcross/intercross\n";
	print "	scr is the screen: either 'm' or 's' \n";
	print "	- recessive mutation or recessive suppressor\n";
	print "	fix is either 'n' or 'r'  - fix number of G3 pups/male or pups/female\n";
	print "	d is the number of G1 pairs per G0 pair\n";
	print "	h is the number of G2 males per G1 par (affects the IC only)\n";
	print "	k is the number of G2 females mated with each G1 male\n";
	print "	n is the number of G3 pups per male/female (depending on what was fixed)\n";
	print "	the -s option includes simulation results,\n";
	print "	 where reps is the number of times the screen is simulated\n";
	print "	 the default is '-s:0' which means no simulations\n";
	print "	the -x option sets the expected number of mutations per G1 pup\n";
	print "	 (default = 100)\n";
	print "	the -y option specifies the minimum number of G3 homozygotes that must\n";
	print "	be produced before a mutation has been 'screened' (default = 1)\n";
	print "Example:\nperl EffiSim.pl -p:g,bc,m,r,2,1,4,20 -n:output.txt -o:l\n";
	print "\nJeremy Silver (silver\@wehi.edu.au), December 2005\n\n";
	exit(1);
}

######################################## Theoretical calculation of the efficiency

sub Calculate{
	my ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @_;
	if($fixed eq "n"){
		@BCLitterProbs = (0) x ($n+1);
		@BCBoxProbs = (0) x ($n*$k+1);
		for($t = 0; $t<=$n; $t++){
			$BCLitterProbs[$t] = MutBCn($n,$t);
			$BCBoxProbs[$t] = $BCLitterProbs[$t];
		}
		@BCBoxProbs = @BCLitterProbs;
		for($litter = 2; $litter<=$k; $litter++){
			@LastProbVec = @BCBoxProbs;
			@BCBoxProbs = (0) x ($n*$k+1);
			for($t = 0; $t<=($litter-1)*$n ; $t++){		# $t = # of affected pups in previous litters
				for($i = 0; $i<=$n; $i++){			# $i = # of affected pups in this litter
					$BCBoxProbs[$t+$i] += $LastProbVec[$t]*$BCLitterProbs[$i];
				}
			}
		}
		$sum = 0;
		foreach $element (@BCBoxProbs){
			$sum += $element;
		}
	} else {
		@BCBoxProbs = (0) x ($n+1);
		for($t = 0; $t<=$n; $t++){
			$BCBoxProbs[$t] = MutBCr($n,$t,$k);
		}
		$sum = 0;
		foreach $element (@BCBoxProbs){
			$sum += $element;
		}
	}
	if($BreedingScheme eq "bc"){
		if($screen eq "m"){
			return(@BCBoxProbs);
		} else {
			@SupBCBoxProbs = (0) x ($#BCBoxProbs+1);
			for($t = 0; $t<=$#BCBoxProbs;$t++){	# Number of m/m,s/s G3s
				for($i = $t; $i<=$#BCBoxProbs;$i++){	# Number of m/m G3s
					$SupBCBoxProbs[$t] += (factorial($i)/(factorial($i-$t)*factorial($t)))*(0.25**$t)*(0.75**($i-$t))*$BCBoxProbs[$i];
				}
			}
			return(@SupBCBoxProbs);
		}
	} else { # Now for the IC mutation screen
		@ICBoxProbs = (0) x ($#BCBoxProbs+1);
		@ICmultiBoxProbs = (0) x ($#BCBoxProbs*$h+1);
		$ICBoxProbs[0] = 0.5;
		for($t = 0; $t<=$#BCBoxProbs; $t++){
			$ICBoxProbs[$t] += 0.5*$BCBoxProbs[$t];
			$ICmultiBoxProbs[$t] = $ICBoxProbs[$t];
		}
		for($G2boxes = 2; $G2boxes <= $h; $G2boxes++){
			@LastProbVec = @ICmultiBoxProbs;
			@ICmultiBoxProbs = (0) x ($#BCBoxProbs*$h+1);
			for($t = 0; $t <= ($G2boxes - 1)*$#BCBoxProbs;$t++){ 	# $t = # of affected pups from the last box
				for($i = 0; $i<=$#ICBoxProbs; $i++){		      	# $i = # of affected pups from this box
					$ICmultiBoxProbs[$t+$i] += $LastProbVec[$t]*$ICBoxProbs[$i];
				}
			}
		}
		if($screen eq "m"){
			return(@ICmultiBoxProbs);
		} else {
			@SupICmultiBoxProbs = (0) x ($#BCBoxProbs+1);
			for($t = 0; $t<=$h*$#BCBoxProbs;$t++){	# Number of m/m,s/s G3s
				for($i = $t; $i<=$h*$#BCBoxProbs;$i++){	# Number of m/m G3s
					$SupICmultiBoxProbs[$t] += (factorial($i)/(factorial($i-$t)*factorial($t)))*(0.25**$t)*(0.75**($i-$t))*$ICmultiBoxProbs[$i];
				}
			}
			return(@SupICmultiBoxProbs);
		}
	}
}

sub MutBCr{

	my $prob = 0;
	my ($n,$t,$k) = @_;
	my $j;
	for($j = 0; $j<=$k; $j++){
		$prob += (factorial($k)/(factorial($k-$j)*factorial($j)))*($j**$t)*((4*$k - $j)**($n-$t));
	}
	$prob *= (0.5**$k)*((1/(4*$k))**$n)*(factorial($n)/(factorial($n-$t)*factorial($t)));
	return $prob;
}

sub MutBCn{

	my $prob = 0;
	my ($n,$t) = @_;
	if($t==0){
		$prob = (0.75**$n)*0.5+0.5;
	} else {
		$prob = (factorial($n)/(factorial($n-$t)*factorial($t)))*(0.25**$t)*(0.75**($n-$t))*0.5;
	}
	return $prob;
}

sub factorial{

	my $n = $_[0];
	my $answer = 1;
	if($n>1){
		my @factors = (2..$n);
		foreach $element (@factors){
			$answer *= $element;
		}
	}
	return $answer;
}

sub simple{

	my ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @_;
	my $mean;
	my $var;
	$x = $Rate;
	
	if($screen eq "m"){
		if($fixed eq "n"){
			($mean,$var) = simpleMutICn($x,$d,$h,$k,$n,$BreedingScheme);
		} else {
			($mean,$var) = simpleMutICr($x,$d,$h,$k,$n,$BreedingScheme);
		}
	} else {
		if($fixed eq "n"){
			($mean,$var) = simpleSupICn($x,$d,$h,$k,$n,$BreedingScheme);
		} else {
			($mean,$var) = simpleSupICr($x,$d,$h,$k,$n,$BreedingScheme);
		}
	}
	return ($mean,$var);
}

sub simpleMutICr{

	my $prob = 0;
	my ($x,$d,$h,$k,$r,$BreedingScheme) = @_;
	my $j;
	my $mean;
	my $var;
	for($j = 0; $j<=$k; $j++){
		$prob += (factorial($k)/(factorial($k-$j)*factorial($j)))*((4*$k - $j)**$r);
	}
	if($BreedingScheme eq "bc"){
		$prob = 0.5 - ($prob*(0.5**($k + 1))*((1/(4*$k))**$r));
		$mean = $x*$d*2*$prob;
		$var = $x*$d*(2*$prob)*(1-(2*$prob));
	} else {
		$prob = 1 - (($prob*(0.5**($k + 1))*((1/(4*$k))**$r)) + 0.5)**$h;
		$mean = 2*$x*$d*$prob;
		$var = 2*$x*$d*$prob*(1-$prob);
	}
	return ($mean,$var);
}

sub simpleMutICn{

	my $prob = 0;
	my $mean;
	my $var;
	my ($x,$d,$h,$k,$n,$BreedingScheme) = @_;
	if($BreedingScheme eq "bc"){
		$prob = 0.5 - (0.5*((0.75**$n)*0.5+0.5)**$k);
		$mean = $x*$d*2*$prob;
		$var = $x*$d*(2*$prob)*(1-(2*$prob));
	} else {
		$prob = 1 - (0.5*((0.75**$n)*0.5+0.5)**$k + 0.5)**$h;
		$mean = 2*$x*$d*$prob;
		$var = 2*$x*$d*$prob*(1-$prob);
	}	
	return ($mean,$var);
}

sub simpleSupICr{

	my ($x,$d,$h,$k,$r,$BreedingScheme) = @_;
	my $i;
	my $j;
	my $preprob;
	my $preprobbc;
	my $prebcEsquared = 0;
	my $preicEsquared1 = 0;
	my $preicEsquared2 = 0;
	my $prob = 0;
	my $Esquared = 0;
	my $mean;
	my $var;
	for($j = 0; $j<=$r; $j++){
		$preprob = 0;
		$preprobbc = 0;
		for($i = 0; $i<=$k; $i++){
			$preprob += (factorial($k)/(factorial($k-$i)*factorial($i)))*(((4*$k - $i)/(12*$k))**$j);			
			$preprobbc += (factorial($k)/(factorial($k-$i)*factorial($i)))*(((4*$k - $i)/(4*$k))**$j);
		}
		$prob += (factorial($r)/(factorial($r-$j)*factorial($j)))*$preprob;
		$prebcEsquared += (factorial($r)/(factorial($r-$j)*factorial($j)))*(0.25**$j)*(0.75**($r - $j))*((1 - (0.5**$k)*$preprobbc)**2);
		$preicEsquared1 += (factorial($r)/(factorial($r-$j)*factorial($j)))*(0.25**$j)*(0.75**($r - $j))*(0.5 + (0.5**($k + 1))*$preprobbc);
		$preicEsquared2 += (factorial($r)/(factorial($r-$j)*factorial($j)))*(0.25**$j)*(0.75**($r - $j))*((0.5 + (0.5**($k+1))*$preprobbc)**2);
	}
	if($BreedingScheme eq "bc"){
		$prob = 1 - (0.5 + (0.5**($k+1))*(0.75**$r)*$prob);
		$mean = $x*$d*2*$prob;
		$Esquared = $x*($x - 1)*$prebcEsquared + $x*2*$prob;
		$var = $d*($Esquared - (($x*2*$prob)**2));
	} else {
		$prob = 1 - ((0.5 + (0.5**($k+1))*(0.75**$r)*$prob)**$h);
		$mean = 2*$x*$d*$prob;
		$Esquared = 2*$x*(2*$x - 1)*(1 - (2*($preicEsquared1**$h)) + ($preicEsquared2**$h)) + (2*$x*$prob);
		$var = $d*($Esquared - (($x*2*$prob)**2));
	}
	return ($mean,$var);
}

sub simpleSupICn{

	my $prob = 0;
	my ($x,$d,$h,$k,$n,$BreedingScheme) = @_;
	my $mean;
	my $var;
	my $Esquared;
	my $preprob = 0;
	if($BreedingScheme eq "bc"){
		$prob = 1 - (0.5*((0.9375**$n)*0.5+0.5)**$k + 0.5);
		$mean = $x*$d*2*$prob;
		for($j = 0; $j<=$n; $j++){
			$preprob += (factorial($n)/(factorial($n-$j)*factorial($j)))*(0.25**$j)*(0.75**($n - $j))*((0.5*(0.75**$j) + 0.5)**2);
		}
		$Esquared = $x*($x - 1)*(1 - 2*((0.5*(0.9375**$n) + 0.5)**$k) + ($preprob**$k)) + $x*2*$prob;
		$var = $d*($Esquared - (($x*2*$prob)**2));
	} else {
		$prob = 1 - (0.5*((0.9375**$n)*0.5+0.5)**$k + 0.5)**$h;
		$mean = 2*$x*$d*$prob;
		for($j = 0; $j<=$n; $j++){
			$preprob += (factorial($n)/(factorial($n-$j)*factorial($j)))*(0.25**$j)*(0.75**($n - $j))*
			((0.5*(0.75**$j) + 0.5)**2);
		}
		$Esquared = 2*$x*(2*$x - 1)*(1 - 2*((0.5*((0.5*(0.9375**$n) + 0.5)**$k) + 0.5)**$h) + ((0.25*($preprob**$k) + 0.5*((0.5*(0.9375**$n) + 0.5)**$k) + 0.25)**$h)) + 2*$x*$prob;
		$var = $d*($Esquared - (($x*2*$prob)**2));
	}
	return ($mean,$var);
}

sub CountMice{
	my ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @_[0..6];
	my @weights = @{$_[7]};
	my $mousecount;
	if($BreedingScheme eq "bc"){
		if($fixed eq "n"){
			$mousecount = 2*$d*$weights[0] + 2*$k*$d*$weights[1] + $d*$k*$n*$weights[2];
		} else {
			$mousecount = 2*$d*$weights[0] + 2*$k*$d*$weights[1] + $d*$n*$weights[2];
		}
	} else {
		if($fixed eq "n"){
			$mousecount = 2*$d*$weights[0] + $d*$h*2*$k*$weights[1] + $d*$h*$n*$k*$weights[2];
		} else {
			$mousecount = 2*$d*$weights[0] + $d*$h*2*$k*$weights[1] + $d*$h*$n*$weights[2];
		}
	}
	return $mousecount;
}

sub GoodnessOfFit{
	$x = $Rate;
	my @probs = @{$_[0]};
	my @count = @{$_[1]};
	my ($BreedingScheme,$screen,$fixed,$d,$h,$k,$n) = @{$_[2]};
	my $factor;
	if($BreedingScheme eq "bc"){
		$factor = $repeats*$d*$x;
	} else {
		$factor = $repeats*$d*2*$x;
	}
	my $i;
	my $sum = 0;
	my $remaindersum = 0;
	my $remaindercount = 0;
	my $remainderprob = 0;
	my $DegreesOfFreedom;
	my $pvalue;
	my $truncated = 0;
	for($i = 0; $i < @probs; $i++){
		if($probs[$i]*$factor > 5){
			$sum += (($count[$i] - $probs[$i]*$factor)**2)/($probs[$i]*$factor);
		} else {
			$truncated = 1;
			$DegreesOfFreedom = $i + 1;
			$remaindercount += $count[$i];
			$remainderprob += $probs[$i];
			$i = @probs;
		}
	}
	if($truncated ==1){
		for($i = $DegreesOfFreedom; $i < @probs; $i++){
			$remaindercount += $count[$i];
			$remainderprob += $probs[$i];
		}
	} else {
		$DegreesOfFreedom = $#probs;
	}
	if($remainderprob > 0){
		$sum += (($remaindercount - $remainderprob*$factor)**2)/($remainderprob*$factor);
	} else {
		$DegreesOfFreedom = @probs - 1;
	}
	return($sum, $DegreesOfFreedom);
}

